DEBUG = True

INSTALLED_APPS = [
    'wagtailsettings',
    'tests.app',

    'wagtail.wagtailcore',
    'wagtail.wagtailadmin',
    'wagtail.wagtailusers',
    'wagtail.wagtailsites',

    'taggit',
    'compressor',
    'modelcluster',

    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.sql',
    }
}

SECRET_KEY = 'Not very secret'

ROOT_URLCONF = 'tests.app.urls'

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    'wagtail.wagtailcore.middleware.SiteMiddleware',
    'wagtail.wagtailredirects.middleware.RedirectMiddleware',
]

from django.conf import global_settings
TEMPLATE_CONTEXT_PROCESSORS = global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
    'django.core.context_processors.request',
    'wagtailsettings.context_processors.settings',
)

WAGTAIL_SITE_NAME = 'Wagtail settings test app'

import os
STATIC_ROOT = os.path.join(os.path.dirname(__file__), 'static')
STATIC_URL = '/static/'



from django.template import Library
from wagtailsettings.context_processors import SettingsProxy

register = Library()


@register.simple_tag(takes_context=True)
def get_settings(context):
    context['settings'] = SettingsProxy()
    return ''



def user_can_edit_setting_type(user, content_type):
    """ true if user has any permission related to this content type """
    if user.is_active and user.is_superuser:
        return True

    permission_codenames = content_type.permission_set.values_list('codename', flat=True)
    for codename in permission_codenames:
        permission_name = "%s.%s" % (content_type.app_label, codename)
        if user.has_perm(permission_name):
            return True

    return False



from django.test import TestCase, modify_settings
from django.template import Template, Context, RequestContext

from wagtail.tests.utils import WagtailTestUtils
from tests.app.models import TestSetting
from wagtail.wagtailcore.models import Site


class TemplateTestCase(TestCase, WagtailTestUtils):
    def setUp(self):
        default_site = Site.objects.get(is_default_site=True)

        self.test_setting = TestSetting.objects.create(
            title='Site title',
            email='initial@example.com',
            site=default_site)


class TestContextProcessor(TemplateTestCase):

    def render(self, string, context=None):
        template = Template(string)
        context = RequestContext(self.client.get('/test/'), context)
        return template.render(context)

    def test_accessing_setting(self):
        self.assertEqual(
            self.render('{{ settings.app.TestSetting.title }}'),
            self.test_setting.title)

    def test_model_case_insensitive(self):
        self.assertEqual(
            self.render('{{ settings.app.testsetting.title }}'),
            self.test_setting.title)
        self.assertEqual(
            self.render('{{ settings.app.TESTSETTING.title }}'),
            self.test_setting.title)
        self.assertEqual(
            self.render('{{ settings.app.TestSetting.title }}'),
            self.test_setting.title)
        self.assertEqual(
            self.render('{{ settings.app.tEstsEttIng.title }}'),
            self.test_setting.title)


@modify_settings(TEMPLATE_CONTEXT_PROCESSORS={
    'remove': 'wagtailsettings.context_processors.settings',
})
class TestTemplateTag(TemplateTestCase):

    def render(self, string, context=None):
        template = Template(string)
        context = Context(context)
        return template.render(context)

    def test_no_context_processor(self):
        self.assertEqual(
            self.render('{{ settings.app.TestSetting.title }}'),
            '')

    def test_accessing_setting(self):
        preamble = '{% load wagtailsettings_tags %}{% get_settings %}'
        self.assertEqual(
            self.render(preamble + '{{ settings.app.TestSetting.title }}'),
            self.test_setting.title)



from django.db.models.loading import get_model
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class SettingsProxy(dict):
    def __init__(self):
        self.cache = {}

    def __getattr__(self, app_label):
        if app_label not in self.cache:
            self.cache[app_label] = SettingModuleProxy(app_label)
        return self.cache[app_label]

    def __str__(self):
        return 'SettingsProxy'


@python_2_unicode_compatible
class SettingModuleProxy(object):
    def __init__(self, app_label):
        self.app_label = app_label
        self.cache = {}

    def __getattr__(self, model_name):
        model_name = model_name.lower()
        if model_name not in self.cache:
            Model = get_model(self.app_label, model_name)
            self.cache[model_name] = Model.objects.first()
        return self.cache[model_name]

    def __str__(self):
        return 'SettingsModuleProxy({0})'.format(self.app_label)


def settings(request):
    return {'settings': SettingsProxy()}



from django.http import Http404
from django.shortcuts import render, redirect
from django.utils.text import capfirst
from django.contrib.contenttypes.models import ContentType
from django.contrib import messages
from django.contrib.auth.decorators import permission_required
from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext as _

from wagtail.wagtailadmin.edit_handlers import ObjectList, extract_panel_definitions_from_model_class

from wagtailsettings.models import get_setting_content_types
from wagtailsettings.permissions import user_can_edit_setting_type


# == Helper functions ==


def get_content_type_from_url_params(app_name, model_name):
    """
    retrieve a content type from an app_name / model_name combo.
    Throw Http404 if not a valid setting type
    """
    try:
        content_type = ContentType.objects.get_by_natural_key(app_name, model_name)
    except ContentType.DoesNotExist:
        raise Http404
    if content_type not in get_setting_content_types():
        # don't allow people to hack the URL to edit content types that aren't registered as settings
        raise Http404

    return content_type


SETTING_EDIT_HANDLERS = {}


def get_setting_edit_handler(model):
    if model not in SETTING_EDIT_HANDLERS:
        panels = extract_panel_definitions_from_model_class(model, ['site'])
        edit_handler = ObjectList(panels).bind_to_model(model)

        SETTING_EDIT_HANDLERS[model] = edit_handler

    return SETTING_EDIT_HANDLERS[model]


# == Views ==


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def edit(request, app_name, model_name):
    content_type = get_content_type_from_url_params(app_name, model_name)
    if not user_can_edit_setting_type(request.user, content_type):
        raise PermissionDenied

    model = content_type.model_class()
    setting_type_name = model._meta.verbose_name

    instance = model.for_site(request.site)
    edit_handler_class = get_setting_edit_handler(model)
    form_class = edit_handler_class.get_form_class(model)

    if request.POST:
        form = form_class(request.POST, request.FILES, instance=instance)

        if form.is_valid():
            form.save()

            messages.success(
                request,
                _("{setting_type} updated.").format(
                    setting_type=capfirst(setting_type_name),
                    instance=instance
                )
            )
            return redirect('wagtailsettings_edit', app_name, model_name)
        else:
            messages.error(request, _("The setting could not be saved due to errors."))
            edit_handler = edit_handler_class(instance=instance, form=form)
    else:
        form = form_class(instance=instance)
        edit_handler = edit_handler_class(instance=instance, form=form)

    return render(request, 'wagtailsettings/edit.html', {
        'content_type': content_type,
        'setting_type_name': setting_type_name,
        'instance': instance,
        'edit_handler': edit_handler,
    })



from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models

from wagtail.wagtailcore import hooks
from wagtail.wagtailadmin.menu import MenuItem

SETTING_MODELS = []

SETTING_CONTENT_TYPES = None


def get_setting_content_types():
    global SETTING_CONTENT_TYPES
    if SETTING_CONTENT_TYPES is None:
        SETTING_CONTENT_TYPES = [
            ContentType.objects.get_for_model(model)
            for model in SETTING_MODELS
        ]

    return SETTING_CONTENT_TYPES


def register_setting(model=None, **kwargs):
    if model is None:
        return lambda model: register_setting(model, **kwargs)

    if model not in SETTING_MODELS:
        SETTING_MODELS.append(model)

    @hooks.register('register_settings_menu_item')
    def hook():
        return MenuItem(
            model._meta.verbose_name.title(),
            reverse('wagtailsettings_edit', args=[
                model._meta.app_label, model._meta.model_name]),
            **kwargs)

    return model


class BaseSetting(models.Model):
    site = models.ForeignKey('wagtailcore.Site', unique=True, db_index=True,
                             editable=False)

    class Meta:
        abstract = True

    @classmethod
    def for_site(cls, site):
        instance, created = cls.objects.get_or_create(site=site)
        return instance



from django.db import models

from wagtailsettings import BaseSetting, register_setting


@register_setting
class TestSetting(BaseSetting):
    title = models.CharField(max_length=100)
    email = models.EmailField()



from django.test import TestCase
from django.core.urlresolvers import reverse

from wagtail.tests.utils import WagtailTestUtils
from tests.app.models import TestSetting
from wagtail.wagtailcore.models import Site


class TestSettingCreateView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

    def get(self, params={}):
        return self.client.get(
            reverse('wagtailsettings_edit', args=('app', 'testsetting')),
            params)

    def post(self, post_data={}):
        return self.client.post(
            reverse('wagtailsettings_edit', args=('app', 'testsetting')),
            post_data)

    def test_status_code(self):
        self.assertEqual(self.get().status_code, 200)

    def test_edit_invalid(self):
        response = self.post(post_data={'foo': 'bar'})
        self.assertContains(response, "The setting could not be saved due to errors.")
        self.assertContains(response, "This field is required.")

    def test_edit(self):
        response = self.post(post_data={'title': 'Edited site title',
                                        'email': 'test@example.com'})
        self.assertEqual(response.status_code, 302)

        default_site = Site.objects.get(is_default_site=True)
        setting = TestSetting.objects.get(site=default_site)
        self.assertEqual(setting.title, 'Edited site title')
        self.assertEqual(setting.email, 'test@example.com')


class TestSettingEditView(TestCase, WagtailTestUtils):
    def setUp(self):
        default_site = Site.objects.get(is_default_site=True)

        self.test_setting = TestSetting()
        self.test_setting.title = 'Site title'
        self.test_setting.email = 'initial@example.com'
        self.test_setting.site = default_site
        self.test_setting.save()

        self.login()

    def get(self, params={}):
        return self.client.get(
            reverse('wagtailsettings_edit', args=('app', 'testsetting')),
            params)

    def post(self, post_data={}):
        return self.client.post(
            reverse('wagtailsettings_edit', args=('app', 'testsetting')),
            post_data)

    def test_status_code(self):
        self.assertEqual(self.get().status_code, 200)

    def test_non_existant_model(self):
        response = self.client.get(
            reverse('wagtailsettings_edit', args=('tests', 'foo')))
        self.assertEqual(response.status_code, 404)

    def test_edit_invalid(self):
        response = self.post(post_data={'foo': 'bar'})
        self.assertContains(response, "The setting could not be saved due to errors.")
        self.assertContains(response, "This field is required.")

    def test_edit(self):
        response = self.post(post_data={'title': 'Edited site title',
                                        'email': 'test@example.com'})
        self.assertEqual(response.status_code, 302)

        default_site = Site.objects.get(is_default_site=True)
        setting = TestSetting.objects.get(site=default_site)
        self.assertEqual(setting.title, 'Edited site title')
        self.assertEqual(setting.email, 'test@example.com')



from django.conf.urls import patterns, include, url

from wagtail.wagtailcore import urls as wagtail_urls
from wagtail.wagtailadmin import urls as wagtailadmin_urls


urlpatterns = patterns(
    '',
    url(r'^admin/', include(wagtailadmin_urls)),
    url(r'', include(wagtail_urls)),
)



#!/usr/bin/env python
"""
Install wagtailsettings using setuptools
"""

from setuptools import setup, find_packages

with open('README.rst', 'r') as f:
    readme = f.read()

with open('wagtailsettings/version.py') as v:
    version = '0.0.0'
    exec(v.read())  # Get version


setup(
    name='wagtailsettings',
    version=version,
    description='Admin-editable settings for Wagtail projects',
    long_description=readme,
    author='Tim Heap',
    author_email='tim@takeflight.com.au',
    url='https://bitbucket.org/takeflight/wagtailsettings',

    install_requires=['wagtail>=0.9'],
    zip_safe=False,
    license='BSD License',

    packages=find_packages(),

    include_package_data=True,
    package_data={},

    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
    ],
)
